{% load i18n %}
{% load navactive %}
{% load aa_forum_template_variables %}

<li class="{% navactive request 'aa_forum:profile_index' %}">
    <a href="{% url 'aa_forum:profile_index' %}">{% translate "Profile" %}</a>
</li>

<li class="{% navactive request 'aa_forum:personal_messages_inbox' %}">
    <a href="{% url 'aa_forum:personal_messages_inbox' %}">
        {% translate "Messages" %}
        {% personal_message_unread_count request.user %}
    </a>
</li>
