{% load i18n %}
{% load navactive %}
{% load aa_forum_template_variables %}

{% set_template_variable url_name = request.resolver_match.url_name %}

<li class="aa-forum-admin-menu dropdown{% if "admin_" in url_name %} active{% endif %}">
    <a class="dropdown-toggle" data-toggle="dropdown" href="#">{% translate "Administration" %}</a>

    <ul id="aa-forum-admin-menu" class="dropdown-menu clearfix" role="menu">
        <li class="aa-forum-admin-menu-item {% navactive request 'aa_forum:admin_categories_and_boards' %}">
            <a href="{% url 'aa_forum:admin_categories_and_boards' %}">{% translate "Categories and Boards" %}</a>
        </li>

        <li class="aa-forum-admin-menu-item {% navactive request 'aa_forum:admin_forum_settings' %}">
            <a href="{% url 'aa_forum:admin_forum_settings' %}">{% translate "Forum Settings" %}</a>
        </li>
    </ul>
</li>
