{% load bootstrap %}
{% load i18n %}

<div class="panel panel-default">
    <div class="panel-heading">
        <div class="panel-title">
            {% translate "Forum Settings" %}
        </div>
    </div>

    <div class="panel-body">
        <form id="aa-forum-form-settings-modify" class="form-horizontal" autocomplete="off" action="{% url 'aa_forum:admin_forum_settings' %}" method="post">
            {% csrf_token %}
            {{ form|bootstrap_horizontal }}

            {% include "aa_forum/partials/form/required-field-hint.html" with form_horizontal=True %}

            <div class="form-group aa-forum-form-group aa-forum-form-modify-settings text-right clearfix">
                <div class="col-xs-12">
                    <button class="btn btn-primary btn-sm" type="submit">
                        {% translate "Submit" %}
                    </button>
                </div>
            </div>
        </form>
    </div>
</div>
