{% extends "aa_forum/base.html" %}

{% load i18n %}

{% block page_title %}{{ board.name }} » {% translate "Forum" %}{% endblock %}

{% block aa_forum_header %}
    {% include "aa_forum/partials/menu.html" %}
    {% include "aa_forum/partials/breadcrumb.html" with category=board.category board=board %}
{% endblock %}

{% block aa_forum_body %}
    {% if board %}
        {% include "aa_forum/partials/forum/board/board-index.html" %}
    {% else %}
        {% include "aa_forum/partials/forum/board/no-access.html" %}
    {% endif %}
{% endblock %}

{% block extra_css %}
    {% include "aa_forum/bundles/aa-forum-css.html" %}
{% endblock %}
