{% extends "aa_forum/base.html" %}

{% load i18n %}

{% block page_title %}{% translate "Forum (Administration » Forum Settings)" %}{% endblock %}

{% block aa_forum_header %}
    {% include "aa_forum/partials/menu.html" %}
{% endblock %}

{% block aa_forum_body %}
    {% include "aa_forum/partials/administration/settings-form.html" %}
{% endblock %}

{% block extra_css %}
    {% include "aa_forum/bundles/aa-forum-css.html" %}
{% endblock %}
