# Generated by Django 4.0.7 on 2022-08-24 09:08

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("aa_forum", "0006_reset_default_settings"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="setting",
            name="value",
        ),
        migrations.RemoveField(
            model_name="setting",
            name="variable",
        ),
        migrations.AddField(
            model_name="setting",
            name="default_max_messages",
            field=models.IntegerField(
                default=15,
                help_text="Maximum number of messages per page in the topic view",
                verbose_name="Messages per page",
            ),
        ),
        migrations.AddField(
            model_name="setting",
            name="default_max_topics",
            field=models.IntegerField(
                default=10,
                help_text="Maximum number of topics per page in the board view",
                verbose_name="Topics per page",
            ),
        ),
    ]
