{% extends "aa_forum/base.html" %}

{% load i18n %}

{% block page_title %}{% translate "Modify Message » Forum" %}{% endblock %}

{% block aa_forum_header %}
    {% include "aa_forum/partials/menu.html" %}
    {% include "aa_forum/partials/breadcrumb.html" with category=board.category board=board topic=message.topic modify_message=True %}
{% endblock %}

{% block aa_forum_body %}
    {% include "aa_forum/partials/forum/topic/modify-message.html" %}
{% endblock %}

{% block extra_css %}
    {% include "aa_forum/bundles/aa-forum-css.html" %}
{% endblock %}

{% block extra_javascript %}
    {% include "aa_forum/bundles/ckeditor-js.html" %}
{% endblock %}
